# withComponentState

higher order component that will automatically create state for your redux
components, and give you the prop `componentState` as well as the prop
`setState` which works like this.setState on class components.

now using redux state is just as easy as using component local state, but with
the added benefit of serialise/restore and time travel.

usage:

```js
// store.js
import {createStore, combineReducers} from 'redux'
import createComponentState from 'with-component-state'
export const store = createStore(combineReducers(reducers))
export const withComponentState = createComponentState(store, reducers)

// meanwhile in FlamingoContainer.js
import React from 'react'

const initialState = {
  sausage: 10,
  bird: 'squerk!',
  dog: '🐶'
}

const FlamingoContainer = ({
  componentState: { sausage, bird, dog },
  setState
}) => (
  <div onClick={() => setState({sausage: sausage + 1})>
    {sausage} and {bird} and {dog}
  </div>
)

export default withComponentState(initialState)(FlamingoContainer)
```

in combination with connect:

```js
import {compose} from 'redux'
import {connect} from 'react-redux'

// …

export default compose(
  connect(mapStateToProps)
  withComponentState({pls: 'halp'})
)(TheCompleteLossOrAbsenceOfHope)
```

thanks

thanks everybody

## todo

* write better api (get access to store without being passed it)
* try and find a way around using `this.forceUpdate()`
* give this a better name. something memorable like pancakeHospital
* fix strange behaviour when changing IDs
* dress up as santa claus
* hide in the shed until christmas
