'use strict';

var redux = require('redux');
var react = require('react');

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();





var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};



var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};











var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

function createReducer(initialState, handlers) {
	return function reducer() {
		var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
		var action = arguments[1];

		return handlers[action.type] ? handlers[action.type](state, action) : state;
	};
}

function createHandlers(action) {
	return defineProperty({}, action, function (state, action) {
		return Object.assign({}, state, action.state);
	});
}

function getDisplayName(component) {
	return component.displayName || component.name || 'MysteryComponent';
}

function createComponentState(store, reducers) {
	var componentReducers = {};
	var reducer = '@components';
	var actionSuffix = 'set state';

	function createActionName(_ref2) {
		var name = _ref2.name,
		    id = _ref2.id;

		return reducer + '/' + name + (id ? '@' + id : '') + '/' + actionSuffix;
	}

	function combine() {
		var combined = Object.assign({}, componentReducers);
		// TODO this might well be a call for reduce-object
		for (var name in combined) {
			if (combined[name].id) {
				combined[name].id = redux.combineReducers(combined[name].id);
			}
			combined[name] = redux.combineReducers(combined[name]);
		}
		reducers[reducer] = redux.combineReducers(combined);
		return redux.combineReducers(reducers);
	}

	function addComponentReducer(_ref3) {
		var initialState = _ref3.initialState,
		    handlers = _ref3.handlers,
		    name = _ref3.name,
		    id = _ref3.id;

		var reducer = createReducer(initialState, handlers);
		componentReducers[name] = componentReducers[name] || {};
		if (id) {
			componentReducers[name].id = componentReducers[name].id || {};
			componentReducers[name].id[id] = reducer;
		} else {
			componentReducers[name].default = reducer;
		}
		store.replaceReducer(combine());
	}

	function removeComponentReducer(_ref4) {
		var name = _ref4.name,
		    id = _ref4.id;

		var reducer = componentReducers[name];
		if (id) {
			delete reducer.id[id];
		} else {
			delete reducer.default;
		}
		store.replaceReducer(combine());
	}

	return function withComponentState(initialState) {
		return function (WrappedComponent) {
			var name = getDisplayName(WrappedComponent);
			var action = void 0;
			var handlers = void 0;
			return function (_Component) {
				inherits(_class, _Component);

				function _class(props) {
					classCallCheck(this, _class);

					var _this = possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).call(this, props));

					var __id = props.__id;

					action = createActionName({ name: name, id: __id });
					handlers = createHandlers(action);
					addComponentReducer({ initialState: initialState, handlers: handlers, name: name, id: __id });
					return _this;
				}

				createClass(_class, [{
					key: 'componentWillUnmount',
					value: function componentWillUnmount() {
						var __id = this.props.__id;

						removeComponentReducer({ name: name, id: __id });
					}
				}, {
					key: 'render',
					value: function render() {
						var _this2 = this;

						var props = Object.assign({}, this.props);
						var __id = props.__id;

						var state = store.getState()[reducer][name];
						var componentState = Object.assign({}, __id ? state.id[__id] : state.default);
						return react.createElement(WrappedComponent, Object.assign({}, props, {
							componentState: componentState,
							setState: function setState(state) {
								_this2.forceUpdate();
								return store.dispatch({
									type: action,
									state: state
								});
							}
						}));
					}
				}]);
				return _class;
			}(react.Component);
		};
	};
}

module.exports = createComponentState;
//# sourceMappingURL=index.js.map
