import {
	combineReducers
} from 'redux'

import {Component, createElement} from 'react'

function createReducer(initialState, handlers) {
	return function reducer(state = initialState, action) {
		return handlers[action.type] ? handlers[action.type](state, action) : state
	}
}

function createHandlers(action) {
	return {
		[action](state, action) {
			return Object.assign({}, state, action.state)
		}
	}
}

function getDisplayName(component) {
	return component.displayName || component.name || 'MysteryComponent'
}

export default function createComponentState(store, reducers) {
	const componentReducers = {}
	const reducer = '@components'
	const actionSuffix = 'set state'

	function createActionName({name, id}) {
		return `${reducer}/${name}${id ? `@${id}` : ''}/${actionSuffix}`
	}

	function combine() {
		const combined = Object.assign({}, componentReducers)
		// TODO this might well be a call for reduce-object
		for (const name in combined) {
			if (combined[name].id) {
				combined[name].id = combineReducers(combined[name].id)
			}
			combined[name] = combineReducers(combined[name])
		}
		reducers[reducer] = combineReducers(combined)
		return combineReducers(reducers)
	}

	function addComponentReducer({ initialState, handlers, name, id }) {
		const reducer = createReducer(initialState, handlers)
		componentReducers[name] = componentReducers[name] || {}
		if (id) {
			componentReducers[name].id = componentReducers[name].id || {}
			componentReducers[name].id[id] = reducer
		} else {
			componentReducers[name].default = reducer
		}
		store.replaceReducer(combine())
	}

	function removeComponentReducer({name, id}) {
		const reducer = componentReducers[name]
		if (id) {
			delete reducer.id[id]
		} else {
			delete reducer.default
		}
		store.replaceReducer(combine())
	}

	return function withComponentState(initialState) {
		return WrappedComponent => {
			const name = getDisplayName(WrappedComponent)
			let action
			let handlers
			return class extends Component {
				constructor(props) {
					super(props)
					const {__id} = props
					action = createActionName({name, id: __id})
					handlers = createHandlers(action)
					addComponentReducer({initialState, handlers, name, id: __id})
				}

				componentWillUnmount() {
					const {__id} = this.props
					removeComponentReducer({name, id: __id})
				}

				render() {
					const props = Object.assign({}, this.props)
					const {__id} = props
					const state = store.getState()[reducer][name]
					const componentState = Object.assign({},
						__id ? state.id[__id] : state.default
					)
					return createElement(WrappedComponent,
						Object.assign({}, props, {
							componentState,
							setState: state => {
								this.forceUpdate()
								return store.dispatch({
									type: action,
									state
								})
							}
						})
					)
				}
			}
		}
	}
}
